PROGRAM compute_mstar
  USE cosmo
  USE linearpk
  USE sigma
  USE growth
! A sample program for computing Mstar by solving sigma(Mstar,z)=1.6865.
! June 22, 2011: E.Komatsu
  IMPLICIT none
  double precision :: zout,Mh,lnRh
  double precision :: pi=3.1415926535d0
  integer :: i,iter,imax=50
  double precision, allocatable :: Rtest(:)
  double precision :: zbrent,func
  double precision :: ltest,m1,m2,ml,mu
  external zbrent,func
  character(len=128) :: filename
  integer :: n
  double precision :: zin,g,growthfactor
  external g
! Specify three cosmological parameters
! The data type has been defined in MODULE cosmo.
! These parameters will be used for computing the linear growth factor.
  ode0=0.723d0
  om0=0.277d0
  w=-1d0
! read in and tabulate P(k)
  filename='wmap5baosn_max_likelihood_matterpower_at_z=30.dat'
  n=896 ! # of lines in the file
  zin=30d0
  CALL open_linearpk(filename,n)
! compute and fit sigma^2(R) to Chebyshev polynomials
  CALL compute_sigma2
  CALL close_linearpk
! compute the growth factor
  CALL setup_growth
  print*,'redshift?'
  read*,zout
  growthfactor=g(zout)/g(zin)*(1d0+zin)/(1d0+zout)
! Find initial radius range for zbrent (non-linear equation solver),
! within which Rstar will be found.
  allocate(Rtest(0:imax))
  ! initial guess for Rstar [h^-1 Mpc]
  Rtest(0)=(3d0*1d13/(4d0*pi*om0*2.775d11))**(1d0/3d0) 
  ltest=func(dlog(Rtest(0)),growthfactor)
  if(ltest.le.0d0)then
     do i=1,imax
        Rtest(i)=Rtest(i-1)/2.d0
        ltest=func(dlog(Rtest(i)),growthfactor)
        if (ltest.gt.0d0)then
           m1=dlog(Rtest(i))
           m2=dlog(Rtest(i-1))
           exit
        endif
     enddo
  else
     do i=1,imax
        Rtest(i)=Rtest(i-1)*2.d0
        ltest=func(dlog(Rtest(i)),growthfactor)
        if(ltest.lt.0d0)then
           m1=dlog(Rtest(i))
           m2=dlog(Rtest(i-1))
           exit
        endif
     enddo
  endif
  deallocate(Rtest)
! Lower & upper limits for R, between which Rstar will be found.
  ml=min(m1,m2) 
  mu=max(m1,m2)
! Find Rstar [h^-1 Mpc] using zbrent.
  lnRh=zbrent(func,ml,mu,1d-4,iter,growthfactor)
  Mh=4d0*pi/3d0*(om0*2.775d11)*dexp(3d0*lnRh) ! h^-1 Msun
  print*,'Mstar=',Mh,' h^-1 Msun'
END PROGRAM compute_mstar
!-----------------------------------------------------------------------------
double precision function func(lnRh,growth)
  USE sigma
! zbrent will solve the equation, sigma(Rstar,z)=1.6865, for ln(Rstar).
  IMPLICIT none
  double precision, intent(IN) :: lnRh,growth ! Rh in units of h^-1 Mpc
  real :: chebev,lnsigma2
  lnsigma2=CHEBEV(lnR1,lnR2,c,ndim,real(lnRh)) ! ln(sigma^2)
  func=0.5d0*dble(lnsigma2)+dlog(growth)-dlog(1.6865d0) 
  return
END function func
